package com.loit.modules.fastdfs;

import java.io.IOException;

import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.csource.common.MyException;
import org.csource.fastdfs.ClientGlobal;
import org.csource.fastdfs.TrackerServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.loit.common.config.SpringContextUtil;

/**
 * 
* 
* @ClassName: TrackerServerPool.java
* @Description: TrackerServer 对象池
* @version: v1.0.0
* @author: gaodengyun
* @date: 2019年4月24日 上午9:03:19 
 */
@Component
public class TrackerServerPool {
    /**
     * org.slf4j.Logger
     */
    private static Logger logger = LoggerFactory.getLogger(TrackerServerPool.class);

    /**
     * TrackerServer 配置文件路径
     */
    private static final String FASTDFS_CONFIG_DEV_PATH ="config/config-dev.properties";
    /**
     * TrackerServer 配置文件路径
     */
    private static final String FASTDFS_CONFIG_PROC_PATH ="config/config-proc.properties";
    /**
     * TrackerServer 配置文件路径
     */
    private static final String FASTDFS_CONFIG_TESST_PATH ="config/config-test.properties";

    /**
     * 最大连接数 default 8.
     */
    private static int maxStorageConnection;
    @Value("${fastdfs.max_storage_connection}")
    public void setMaxStorageConnection(int maxStorageConnection) {
		TrackerServerPool.maxStorageConnection = maxStorageConnection;
	}

    /**
     * TrackerServer 对象池.
     * GenericObjectPool 没有无参构造
     */
    private static GenericObjectPool<TrackerServer> trackerServerPool;

    private TrackerServerPool(){};

    private static synchronized GenericObjectPool<TrackerServer> getObjectPool(){
        if(trackerServerPool == null){
            try {
                // 加载配置文件
            	String activeMode = SpringContextUtil.getActiveProfile();
            	System.out.println("activeMode:==="+activeMode);
            	System.out.println("maxStorageConnection:==="+maxStorageConnection);
            	switch (activeMode) {
    			case "proc":
    				ClientGlobal.initByProperties(FASTDFS_CONFIG_PROC_PATH);
    				break;
    			case "dev":
    				ClientGlobal.initByProperties(FASTDFS_CONFIG_DEV_PATH);
    				break;
    			default:
    				ClientGlobal.initByProperties(FASTDFS_CONFIG_TESST_PATH);
    			}
                
            } catch (IOException e) {
                e.printStackTrace();
            } catch (MyException e) {
                e.printStackTrace();
            }

            if(logger.isDebugEnabled()){
                logger.debug("ClientGlobal configInfo: {}", ClientGlobal.configInfo());
            }

            // Pool配置
            GenericObjectPoolConfig poolConfig = new GenericObjectPoolConfig();
            poolConfig.setMinIdle(2);
            if(maxStorageConnection > 0){
                poolConfig.setMaxTotal(maxStorageConnection);
            }

            trackerServerPool = new GenericObjectPool<>(new TrackerServerFactory(), poolConfig);
        }
        return trackerServerPool;
    }

    /**
     * 获取 TrackerServer
     * @return TrackerServer
     * @throws FastDFSException
     */
    public static TrackerServer borrowObject() throws FastDFSException {
        TrackerServer trackerServer = null;
        try {
            trackerServer = getObjectPool().borrowObject();
            System.out.println(trackerServerPool.getBorrowedCount()+","+trackerServerPool.getCreatedCount()+","+trackerServerPool.getDestroyedByBorrowValidationCount()+","+trackerServerPool.getDestroyedByEvictorCount()+","+trackerServerPool.getDestroyedCount()+","+trackerServerPool.getReturnedCount());
        } catch (Exception e) {
            e.printStackTrace();
            if(e instanceof FastDFSException){
                throw (FastDFSException) e;
            }
        }
        return trackerServer;
    }

    /**
     * 回收 TrackerServer
     * @param trackerServer 需要回收的 TrackerServer
     */
    public static void returnObject(TrackerServer trackerServer){
       getObjectPool().returnObject(trackerServer);
    }


}
